"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavigationContext = void 0;
class NavigationContext {
    constructor(theme, options) {
        this.theme = theme;
        this.options = options;
        this.navigation = [];
    }
    getNavigation(project) {
        var _a;
        const entryFileName = this.options.entryFileName;
        const hasReadme = !((_a = this.options.readme) === null || _a === void 0 ? void 0 : _a.endsWith('none'));
        if (hasReadme) {
            this.navigation.push({
                title: 'README',
                url: entryFileName,
            });
            this.navigation.push({
                title: 'API',
                url: project.url,
            });
        }
        else {
            this.navigation.push({
                title: 'API',
                url: entryFileName,
            });
        }
        this.buildNavigationFromProject(project);
        return this.navigation;
    }
    buildNavigationFromProject(project) {
        var _a, _b, _c;
        if ((_a = project.groups) === null || _a === void 0 ? void 0 : _a.length) {
            if (project.groups.length === 1) {
                const children = this.getGroupChildren(project.groups[0]);
                if (children) {
                    this.navigation.push(...children);
                }
            }
            else {
                (_b = project.groups) === null || _b === void 0 ? void 0 : _b.forEach((projectGroup) => {
                    const children = this.getGroupChildren(projectGroup);
                    if (children.length) {
                        this.navigation.push({
                            title: projectGroup.title,
                            children,
                        });
                    }
                });
            }
        }
        else {
            (_c = project.children) === null || _c === void 0 ? void 0 : _c.forEach((child) => {
                this.navigation.push({
                    title: child.name,
                    children: this.getChildrenOrGroups(child) || [],
                    ...(child.url && { url: child.url }),
                });
            });
        }
    }
    getCategoryGroupChildren(group) {
        var _a;
        return (_a = group.children) === null || _a === void 0 ? void 0 : _a.filter((child) => child.hasOwnDocument).map((child) => {
            const children = this.getChildrenOrGroups(child);
            return {
                title: child.name,
                url: child.url,
                ...(children && { children }),
            };
        });
    }
    getGroupChildren(group) {
        var _a, _b, _c;
        if ((_a = group === null || group === void 0 ? void 0 : group.categories) === null || _a === void 0 ? void 0 : _a.length) {
            return (_b = group.categories) === null || _b === void 0 ? void 0 : _b.map((category) => {
                return {
                    title: category.title,
                    children: this.getCategoryGroupChildren(category),
                };
            });
        }
        return (_c = group.children) === null || _c === void 0 ? void 0 : _c.filter((child) => child.hasOwnDocument).map((child) => {
            var _a, _b;
            const mapping = this.theme.getTemplateMapping(child.kind);
            if (Boolean(mapping)) {
                const children = ((_a = child.categories) === null || _a === void 0 ? void 0 : _a.length)
                    ? (_b = child.categories) === null || _b === void 0 ? void 0 : _b.map((category) => {
                        const catChildren = this.getCategoryGroupChildren(category);
                        return catChildren.length
                            ? {
                                title: category.title,
                                children: catChildren,
                            }
                            : null;
                    }).filter((cat) => Boolean(cat))
                    : this.getChildrenOrGroups(child);
                return {
                    title: child.name,
                    url: child.url,
                    ...(children && { children }),
                };
            }
        });
    }
    getChildrenOrGroups(reflection) {
        var _a, _b, _c;
        if ((_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.some((group) => group.allChildrenHaveOwnDocument())) {
            if (this.options.excludeGroups) {
                return (_b = reflection.children) === null || _b === void 0 ? void 0 : _b.filter((child) => child.hasOwnDocument).map((child) => {
                    const children = this.getChildrenOrGroups(child);
                    return {
                        title: child.name,
                        url: child.url,
                        ...(children && { children }),
                    };
                });
            }
            return (_c = reflection.groups) === null || _c === void 0 ? void 0 : _c.map((group) => {
                const groupChildren = this.getGroupChildren(group);
                return groupChildren.length
                    ? {
                        title: group.title,
                        children: this.getGroupChildren(group) || null,
                    }
                    : null;
            }).filter((group) => Boolean(group));
        }
        return null;
    }
}
exports.NavigationContext = NavigationContext;
