"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.header = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
const helpers_1 = require("../../helpers");
/**
 * @category Partials
 */
function header(context, page) {
    const isMonoRepo = !Boolean(page.project.groups);
    if (isMonoRepo) {
        const packageItem = findPackage(page.model);
        if (packageItem) {
            return packageHeader(context, page);
        }
    }
    return projectHeader(context, page);
}
exports.header = header;
function projectHeader(context, page) {
    var _a;
    const entryFileName = context.options.getValue('entryFileName');
    const indexFileName = (0, helpers_1.getIndexFileName)(page.project);
    const titleLink = context.options.getValue('titleLink');
    const hasBreadcrumbs = !context.options.getValue('hideBreadcrumbs');
    const hasReadme = !((_a = context.options.getValue('readme')) === null || _a === void 0 ? void 0 : _a.endsWith('none'));
    const projectName = (0, helpers_1.getProjectDisplayName)(page.project, context.options.getValue('includeVersion'));
    const md = [];
    if (Boolean(titleLink)) {
        md.push((0, elements_1.link)((0, elements_1.bold)(projectName), titleLink));
    }
    else {
        md.push((0, elements_1.bold)(projectName));
    }
    if (hasReadme) {
        md.push((0, elements_1.link)('README', context.relativeURL(entryFileName)));
    }
    if (hasReadme || !hasBreadcrumbs) {
        md.push((0, elements_1.link)('API', hasReadme
            ? context.relativeURL(indexFileName)
            : context.relativeURL(entryFileName)));
    }
    return `${md.join(' ∙ ')}\n\n***\n`;
}
function packageHeader(context, page) {
    const packageItem = findPackage(page.model);
    if (!packageItem) {
        return '';
    }
    const md = [];
    const entryFileName = context.options.getValue('entryFileName');
    const hasReadme = packageItem.readme;
    const hasBreadcrumbs = !context.options.getValue('hideBreadcrumbs');
    md.push((0, elements_1.bold)(packageItem.name));
    if (hasReadme) {
        md.push((0, elements_1.link)('README', entryFileName));
    }
    if (hasReadme || !hasBreadcrumbs) {
        md.push((0, elements_1.link)('API', hasReadme
            ? context.relativeURL(packageItem.url)
            : context.relativeURL(packageItem.url)));
    }
    return `${md.join(' ∙ ')}\n\n***\n`;
}
function findPackage(model) {
    var _a;
    if (model.kindOf(typedoc_1.ReflectionKind.Module) &&
        ((_a = model.parent) === null || _a === void 0 ? void 0 : _a.kindOf(typedoc_1.ReflectionKind.Project))) {
        return model;
    }
    if (model.parent) {
        return findPackage(model.parent);
    }
    return null;
}
