"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declarationMember = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../../../support/elements");
/**
 * @category Partials
 */
function declarationMember(context, declaration, headingLevel, nested = false) {
    var _a, _b, _c, _d, _e, _f, _g;
    const md = [];
    const typeDeclaration = (_a = declaration.type) === null || _a === void 0 ? void 0 : _a.declaration;
    md.push(context.declarationMemberIdentifier(declaration));
    if (declaration.comment) {
        md.push(context.comment(declaration.comment, headingLevel));
    }
    if (declaration.type instanceof typedoc_1.IntersectionType) {
        (_c = (_b = declaration.type) === null || _b === void 0 ? void 0 : _b.types) === null || _c === void 0 ? void 0 : _c.forEach((intersectionType) => {
            if (intersectionType instanceof typedoc_1.ReflectionType) {
                md.push((0, elements_1.heading)(headingLevel, 'Type declaration'));
                md.push(context.typeDeclarationMember(intersectionType.declaration, headingLevel));
            }
        });
    }
    if (declaration.typeParameters) {
        md.push((0, elements_1.heading)(headingLevel, 'Type parameters'));
        md.push(context.typeParametersTable(declaration.typeParameters));
    }
    if (typeDeclaration) {
        if (typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.indexSignature) {
            md.push((0, elements_1.heading)(headingLevel, `Index signature`));
            md.push(context.indexSignatureTitle(typeDeclaration.indexSignature));
        }
        if (((_d = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.signatures) === null || _d === void 0 ? void 0 : _d.length) ||
            ((_e = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.children) === null || _e === void 0 ? void 0 : _e.length)) {
            if ((_f = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.signatures) === null || _f === void 0 ? void 0 : _f.length) {
                typeDeclaration.signatures.forEach((signature) => {
                    md.push(context.signatureMember(signature, headingLevel, true));
                });
            }
            if (!nested && ((_g = typeDeclaration === null || typeDeclaration === void 0 ? void 0 : typeDeclaration.children) === null || _g === void 0 ? void 0 : _g.length)) {
                md.push((0, elements_1.heading)(headingLevel, 'Type declaration'));
                md.push(context.typeDeclarationMember(typeDeclaration, headingLevel));
            }
        }
    }
    md.push(context.inheritance(declaration, headingLevel));
    if (!nested && declaration.sources) {
        md.push(context.sources(declaration, headingLevel));
    }
    return md.join('\n\n');
}
exports.declarationMember = declarationMember;
