"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UrlsContext = void 0;
const path = __importStar(require("path"));
const typedoc_1 = require("typedoc");
const custom_maps_1 = require("../plugin/options/custom-maps");
const url_mapping_1 = require("../plugin/url-mapping");
const utils_1 = require("../support/utils");
const helpers_1 = require("./helpers");
class UrlsContext {
    constructor(theme, options) {
        this.theme = theme;
        this.options = options;
        this.urls = [];
        this.navigation = [];
        this.anchors = {};
    }
    /**
     * Map the models of the given project to the desired output files.
     * Based on TypeDoc DefaultTheme.getUrls()
     *
     * @param project  The project whose urls should be generated.
     */
    getUrls(project) {
        var _a, _b;
        const entryFileName = this.options.entryFileName;
        const indexFileName = (0, helpers_1.getIndexFileName)(project);
        const hasReadme = !((_a = this.options.readme) === null || _a === void 0 ? void 0 : _a.endsWith('none'));
        if (hasReadme) {
            project.url = indexFileName;
            this.urls.push(new url_mapping_1.UrlMapping(entryFileName, project, this.theme.readmeTemplate));
            this.urls.push(new url_mapping_1.UrlMapping(indexFileName, project, this.theme.projectTemplate));
        }
        else {
            project.url = entryFileName;
            this.urls.push(new url_mapping_1.UrlMapping(entryFileName, project, this.theme.projectTemplate));
        }
        if (this.options.entryPointStrategy === typedoc_1.EntryPointStrategy.Packages &&
            !Boolean(project.groups)) {
            (_b = project.children) === null || _b === void 0 ? void 0 : _b.forEach((projectChild) => {
                const url = `${projectChild.name}/${Boolean(projectChild.readme) ? indexFileName : entryFileName}`;
                if (projectChild.readme) {
                    this.urls.push(new url_mapping_1.UrlMapping(`${path.dirname(url)}/${entryFileName}`, projectChild, this.theme.readmeTemplate));
                }
                this.urls.push(new url_mapping_1.UrlMapping(url, projectChild, this.theme.projectTemplate));
                projectChild.url = url;
                this.buildUrlsFromProject(projectChild, url);
            });
        }
        else {
            this.buildUrlsFromProject(project);
        }
        return this.urls;
    }
    /**
     *
     * @param project
     * @param isPackage
     */
    buildUrlsFromProject(project, parentUrl) {
        var _a;
        (_a = project.groups) === null || _a === void 0 ? void 0 : _a.forEach((projectGroup) => {
            var _a;
            (_a = projectGroup.children) === null || _a === void 0 ? void 0 : _a.forEach((projectGroupChild) => {
                this.buildUrlsFromGroup(projectGroupChild, {
                    ...(parentUrl && { parentUrl: parentUrl }),
                });
            });
        });
    }
    buildUrlsFromGroup(reflection, options) {
        var _a;
        const mapping = this.theme.getTemplateMapping(reflection.kind);
        if (mapping) {
            const directory = options.directory || mapping.directory;
            const url = this.getUrl(reflection, {
                ...options,
                directory,
            });
            this.urls.push(new url_mapping_1.UrlMapping(url, reflection, mapping.template));
            reflection.url = url;
            reflection.hasOwnDocument = true;
            (_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.forEach((group) => {
                group.children.forEach((groupChild) => {
                    const mapping = this.theme.getTemplateMapping(groupChild.kind);
                    this.buildUrlsFromGroup(groupChild, {
                        parentUrl: url,
                        directory: (mapping === null || mapping === void 0 ? void 0 : mapping.directory) || null,
                    });
                });
            });
        }
        else if (reflection.parent) {
            this.applyAnchorUrl(reflection, reflection.parent);
        }
    }
    getUrl(reflection, options) {
        const alias = reflection.name.replace(/^_+/, '');
        const parentDir = options.parentUrl
            ? path.dirname(options.parentUrl)
            : null;
        const dir = () => {
            if (reflection.kind === typedoc_1.ReflectionKind.Namespace) {
                return `${options.directory}/${alias}`;
            }
            if (reflection.kind === typedoc_1.ReflectionKind.Module) {
                if (path.parse(this.options.entryFileName).name === alias) {
                    return `module.${alias}`;
                }
                return alias;
            }
            return options.directory
                ? options.directory
                : `${(0, utils_1.slugify)(typedoc_1.ReflectionKind.singularString(reflection.kind))}.${alias}`;
        };
        const filename = () => {
            if ([typedoc_1.ReflectionKind.Module, typedoc_1.ReflectionKind.Namespace].includes(reflection.kind) &&
                this.options.outputFileStrategy === custom_maps_1.OutputFileStrategy.Modules &&
                !this.childrenIncludeNamespaces(reflection)) {
                return null;
            }
            if ([typedoc_1.ReflectionKind.Module, typedoc_1.ReflectionKind.Namespace].includes(reflection.kind)) {
                return path.parse(this.options.entryFileName).name;
            }
            return alias;
        };
        return ([parentDir, dir(), filename()].filter((part) => Boolean(part)).join('/') +
            '.md');
    }
    applyAnchorUrl(reflection, container) {
        var _a, _b;
        if (container.url) {
            if (!reflection.kindOf(typedoc_1.ReflectionKind.TypeLiteral)) {
                const anchorPrefix = this.options.anchorPrefix;
                const anchorId = this.getAnchorId(reflection);
                if (anchorId) {
                    if (!this.anchors[container.url]) {
                        this.anchors[container.url] = [];
                    }
                    this.anchors[container.url].push(anchorId);
                    const count = (_b = (_a = this.anchors[container.url]) === null || _a === void 0 ? void 0 : _a.filter((id) => id === anchorId)) === null || _b === void 0 ? void 0 : _b.length;
                    const anchorParts = [anchorId];
                    if (count > 1) {
                        anchorParts.push(`-${count - 1}`);
                    }
                    if (anchorPrefix) {
                        anchorParts.unshift(`${anchorPrefix}`);
                    }
                    reflection.url = container.url + '#' + anchorParts.join('');
                    reflection.anchor = anchorParts.join('');
                }
            }
            reflection.hasOwnDocument = false;
        }
        if (reflection.parent) {
            reflection.traverse((child) => {
                if (child instanceof typedoc_1.DeclarationReflection) {
                    this.applyAnchorUrl(child, container);
                }
            });
        }
    }
    getAnchorId(reflection) {
        const preserveAnchorCasing = this.options.preserveAnchorCasing;
        const anchorName = this.getAnchorName(reflection);
        if (anchorName) {
            return preserveAnchorCasing ? anchorName : anchorName.toLowerCase();
        }
        return null;
    }
    getAnchorName(reflection) {
        const htmlTableAnchors = this.options.namedAnchors && this.options.namedAnchors['tableRows'];
        if (!htmlTableAnchors) {
            if ((reflection.kindOf(typedoc_1.ReflectionKind.Property) &&
                this.options.propertiesFormat === 'table') ||
                (reflection.kindOf(typedoc_1.ReflectionKind.EnumMember) &&
                    this.options.enumMembersFormat === 'table')) {
                return null;
            }
        }
        if (reflection.kindOf(typedoc_1.ReflectionKind.Constructor)) {
            return 'Constructors';
        }
        return (0, utils_1.slugify)((0, helpers_1.getMemberTitle)(reflection));
    }
    childrenIncludeNamespaces(reflection) {
        var _a;
        return (_a = reflection.children) === null || _a === void 0 ? void 0 : _a.some((child) => child.kind === typedoc_1.ReflectionKind.Namespace);
    }
}
exports.UrlsContext = UrlsContext;
